#include "TCPSocket.h"

#ifdef WIN32
#	include <winsock.h>
#else
#	define INVALID_SOCKET	-1
#	define SOCKET_ERROR		-1
#	include <netinet/in.h>
#	include <arpa/inet.h>
#endif

#include <iostream>

using namespace std;

TCPSocket::TCPSocket() {
	
#ifdef WIN32
	WSADATA info;
	WSAStartup(MAKEWORD(2, 0), &info);
#endif
	
	socketFileDescriptor = ::socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);	
	socketCreated = socketFileDescriptor != INVALID_SOCKET;
	
	if (socketCreated) {
		
		cout << "TCP socket created." << endl;
		
	} else {
		
		cerr << "Failed to create TCP socket." << endl;
	}
	
	socketConnected = false;
	socketSend = false;
	socketReceive = false;
}

void TCPSocket::connect(sockaddr_in * server) {
	
	if (socketCreated && !socketConnected) {
						
		socketConnected = ::connect(socketFileDescriptor, (sockaddr *) server, sizeof(* server)) == 0;
		
		if (socketConnected) {
			
			cout << "Connected to server " << inet_ntoa(server->sin_addr) << ":" << ntohs(server->sin_port) << "." << endl;
			socketSend = true;
			
		} else {
			
			cerr << "Failed to connect to server " << inet_ntoa(server->sin_addr) << ":" << ntohs(server->sin_port) << "." << endl;
		}
	}
}

bool TCPSocket::send(char *msg) {
	
	if (socketSend) {
		
		char data[8193];
		int msgLength = (int) strlen(msg);
		int msgLength2 = htonl(msgLength);
		
		memcpy(data, &msgLength2, sizeof(msgLength2));
		memcpy(data + sizeof(msgLength2), msg, msgLength);
		
		int sentBytes = ::send(socketFileDescriptor, data, sizeof(msgLength2) + msgLength, 0);
		
		socketSend = false;
		socketReceive = true;
		
		switch (sentBytes) {
			
			case SOCKET_ERROR:
				cerr << "Failed to send data." << endl;
				socketReceive = false;
				break;
				
			default:
				if (sentBytes != (int) (sizeof(msgLength2) + msgLength)) {
					
					cerr << "Error sending data. Length of sent data does not match." << endl;
					socketReceive = false;
				}
		}
	}
	
	return socketReceive;
}

bool TCPSocket::receive(char *msg) {
	
	if (socketReceive) {
		
		char data[8193];
		
		int receivedBytes = ::recv(socketFileDescriptor, data, 8192, 0);
		
		socketReceive = false;
		socketSend = true;
		
		switch (receivedBytes) {
		
			case SOCKET_ERROR:
				cerr << "Failed to receive data." << endl;
				msg[0] = '\0';
				socketSend = false;
				break;
				
			case 0: 
				cout << "No data available." << endl;
				msg[0] = '\0';
				socketSend = false;
				break;
				
			default:
				int msgLength2;
				memcpy(&msgLength2, data, sizeof(msgLength2));
				int msgLength = ntohl(msgLength2);
				
				if (receivedBytes != (int) (sizeof(msgLength2) + msgLength)) {
				
					cerr << "Error receiving data. Length of received data does not match." << endl;
					msg[0] = '\0';
					
				} else {
				
					memcpy(msg, data + sizeof(msgLength2), msgLength);
					msg[msgLength] = '\0';
				}
		}
	}
	return socketSend;
}

void TCPSocket::close() {
	
	if (socketCreated) {
		
#ifdef WIN32
		socketCreated = ::closesocket(socketFileDescriptor) == SOCKET_ERROR;
#else
		socketCreated = ::close(socketFileDescriptor) == SOCKET_ERROR;
#endif
		
		if (!socketCreated) {
			
#ifdef WIN32
			WSACleanup();
#endif
			
			cout << "TCP socket closed." << endl;
			socketConnected = false;
			socketSend = false;
			socketReceive = false;
			
		} else {
			
			cerr << "Failed to close TCP socket." << endl;
		}
	}
}
