/*! \file main.cpp
<pre>
<b>File:</b>          main.cpp
<b>Project:</b>       Robocup 3D
<b>Authors:</b>       Hviezdna Jedenastka
<b>Created:</b>       01/08/2008
<b>Contents:</b>	  This is the main part of project
<hr size=2>
<h2><b>Changes</b></h2>
<b>Date</b>             <b>Author</b>          <b>Comment</b>
01/08/2008       Hviezdna Jedenastka           First Version
</pre>
*/

#include <iostream>

#ifdef WIN32
#   include <windows.h>
#endif

#include "Prototype.h"
#include "Communication.h"
#include "tinyxml.h"
#include "TSocket.h"
#include "config.h"

#define XML_CONFIG_FILE "agent.xml"

using namespace std;

int XmlTest();
Config config;

void printHelp() {

	cout << "Prototyp - Prototype player for RoboCup by Hviezdna jedenastka (2007)" << endl << endl;
	cout << "usage: prototyp [arguments]" << endl << endl;
	cout << "Arguments:"<< endl;
	cout << "  -h\t\tPrint Help (this message) and exit" << endl;
	cout << "  -c <config>\tConfig file, default agent.xml" << endl;
	cout << "  --xmltest\tXML test" << endl;
}

int main (int argc, char *argv[])
{
	char *config_file = XML_CONFIG_FILE;
	char message[NET_BUFFER_SIZE + 1];
	
	for (int i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-h") == 0)
		{
			printHelp();
			return 0;
		}
        
		if (strcmp(argv[i], "-c") == 0)
		{
			if (i+1 < argc)
				config_file = argv[i+1];
		}

		if (strcmp(argv[i], "--xmltest") == 0)
		{
			return XmlTest();
		}
	}

	printf("\nUsing config: %s\n", config_file);
	if (config.loadConfig(config_file))
		return 1;

	Prototype prototype;
    
	prototype.player_number = config.player.number;
	prototype.player_team = config.player.team;
	
	if (config.useThreads)
	{
		if (!prototype.init(config.serverIP, config.serverPort))
		{
			printf("Cannot connect to server: %s:%d\n", config.serverIP, config.serverPort);
			return 1;
		}
		printf("Creating communication thread\n");
		if (prototype.run())
			return 1;
	}
	else
	{
		TSocket socket;
		socket.connect(config.serverIP, config.serverPort);
		while (true)
		{
			if (socket.select())
			{
				if (!socket.receive(message, sizeof(message)))
				{
					socket.close();
					return 0;
				}
				printf("Mess: %s\n", message);
			}
			//Sleep(5000);
		}
	}
	
	return 0;
}
