/*
 *  Stack.h
 *  Prototype
 *
 *  Created by Stefan Dlugolinsky on 25.2.2008.
 *  Copyright 2008 Hviezdna jedenastka. All rights reserved.
 *
 */

#ifndef STACK_H_
#define STACK_H_

#include <assert.h>

template<class T>
class Stack {

private:
	int maxsize;
	int count;
	int head;
	int tail;
	T* data;
	
public:
	Stack(int _maxsize);
	~Stack();

	bool empty();
	T& top();
	void pop();
	void push(const T& element);
	int size();
};

template<class T>
Stack<T>::Stack(int _maxsize) {

	assert(_maxsize > 0);

	maxsize = _maxsize;
	count = 0;
	head = tail = -1;
	data = new T[maxsize];
};

template<class T>
Stack<T>::~Stack() {

	delete[] data;
};

template<class T>
bool Stack<T>::empty() {

	return head == -1;
};

template<class T>
T& Stack<T>::top() {

	assert(!empty());
	
	return data[head];
};

template<class T>
void Stack<T>::pop() {

	if (!empty()) {
			
		if (head == tail) {
		
			head = tail = -1;
			
		} else {
	
			head--;
			
			if (head < 0) {
			
				head = maxsize - 1;
			}
		}
		
		count--;
	}
};

template<class T>
void Stack<T>::push(const T& element) {

	if (empty()) {
	
		head = tail = 0;
		
	} else {

		head = (head + 1) % maxsize;
	
		if (head == tail) {
		
			tail = (tail + 1) % maxsize;
			count--;
		}
	}
	
	data[head] = element;
	count++;
};

template<class T>
int Stack<T>::size() {

	return count;
};

#endif /*STACK_H_*/